@@ -21,18 +21,15 @@
 #include "config.h"
 
 #include "ev-document.h"
+
 #include "ev-backend-marshalers.h"
 
 static void ev_document_class_init (gpointer g_class);
 
-enum
-{
-	CHANGED,
-	LAST_SIGNAL
-};
 
-static guint signals[LAST_SIGNAL] = { 0 };
+GMutex *ev_doc_mutex = NULL;
 
+#define LOG(x) 
 GType
 ev_document_get_type (void)
 {
@@ -69,31 +66,42 @@ ev_document_error_quark (void)
 static void
 ev_document_class_init (gpointer g_class)
 {
-	signals[CHANGED] =
-		g_signal_new ("changed",
-			      EV_TYPE_DOCUMENT,
-			      G_SIGNAL_RUN_LAST,
-			      G_STRUCT_OFFSET (EvDocumentIface, changed),
-			      NULL, NULL,
-			      g_cclosure_marshal_VOID__VOID,
-			      G_TYPE_NONE,
-			      0);
-
-	g_object_interface_install_property (g_class,
-				g_param_spec_string ("title",
-						     "Document Title",
-						     "The title of the document",
-						     NULL,
-						     G_PARAM_READABLE));
 }
 
+GMutex *
+ev_document_get_doc_mutex (void)
+{
+	if (ev_doc_mutex == NULL) {
+		ev_doc_mutex = g_mutex_new ();
+	}
+	return ev_doc_mutex;
+}
+
+void
+ev_document_doc_mutex_lock (void)
+{
+	g_mutex_lock (ev_document_get_doc_mutex ());
+}
+
+void
+ev_document_doc_mutex_unlock (void)
+{
+	g_mutex_unlock (ev_document_get_doc_mutex ());
+}
+
+
+
 gboolean
 ev_document_load (EvDocument  *document,
 		  const char  *uri,
 		  GError     **error)
 {
 	EvDocumentIface *iface = EV_DOCUMENT_GET_IFACE (document);
-	return iface->load (document, uri, error);
+	gboolean retval;
+	LOG ("ev_document_load");
+	retval = iface->load (document, uri, error);
+
+	return retval;
 }
 
 gboolean
@@ -102,106 +110,152 @@ ev_document_save (EvDocument  *document,
 		  GError     **error)
 {
 	EvDocumentIface *iface = EV_DOCUMENT_GET_IFACE (document);
-	return iface->save (document, uri, error);
-}
+	gboolean retval;
 
-char *
-ev_document_get_title (EvDocument  *document)
-{
-	char *title;
+	LOG ("ev_document_save");
+	retval = iface->save (document, uri, error);
 
-	g_object_get (document, "title", &title, NULL);
-
-	return title;
+	return retval;
 }
 
 int
 ev_document_get_n_pages (EvDocument  *document)
 {
 	EvDocumentIface *iface = EV_DOCUMENT_GET_IFACE (document);
-	return iface->get_n_pages (document);
-}
+	gint retval;
 
-void
-ev_document_set_page (EvDocument  *document,
-		      int          page)
-{
-	EvDocumentIface *iface = EV_DOCUMENT_GET_IFACE (document);
-	iface->set_page (document, page);
-}
+	LOG ("ev_document_get_n_pages");
+	retval = iface->get_n_pages (document);
 
-int
-ev_document_get_page (EvDocument *document)
-{
-	EvDocumentIface *iface = EV_DOCUMENT_GET_IFACE (document);
-	return iface->get_page (document);
-}
-
-void
-ev_document_set_target (EvDocument  *document,
-			GdkDrawable *target)
-{
-	EvDocumentIface *iface = EV_DOCUMENT_GET_IFACE (document);
-	iface->set_target (document, target);
-}
-
-void
-ev_document_set_scale (EvDocument   *document,
-		       double        scale)
-{
-	EvDocumentIface *iface = EV_DOCUMENT_GET_IFACE (document);
-	iface->set_scale (document, scale);
-}
-
-void
-ev_document_set_page_offset (EvDocument  *document,
-			     int          x,
-			     int          y)
-{
-	EvDocumentIface *iface = EV_DOCUMENT_GET_IFACE (document);
-	iface->set_page_offset (document, x, y);
+	return retval;
 }
 
 void
 ev_document_get_page_size   (EvDocument   *document,
 			     int           page,
-			     int          *width,
-			     int          *height)
+			     double       *width,
+			     double       *height)
 {
 	EvDocumentIface *iface = EV_DOCUMENT_GET_IFACE (document);
+
+	LOG ("ev_document_get_page_size");
 	iface->get_page_size (document, page, width, height);
 }
 
 char *
-ev_document_get_text (EvDocument   *document,
-		      GdkRectangle *rect)
+ev_document_get_page_label(EvDocument    *document,
+			   int             page)
 {
 	EvDocumentIface *iface = EV_DOCUMENT_GET_IFACE (document);
-	return iface->get_text (document, rect);
+
+	LOG ("ev_document_get_page_label");
+	if (iface->get_page_label == NULL)
+		return NULL;
+
+	return iface->get_page_label (document, page);
 }
 
-EvLink *
-ev_document_get_link (EvDocument   *document,
-		      int           x,
-		      int	    y)
+gboolean
+ev_document_can_get_text (EvDocument  *document)
 {
 	EvDocumentIface *iface = EV_DOCUMENT_GET_IFACE (document);
-	return iface->get_link (document, x, y);
+
+	return iface->can_get_text (document);
+}
+
+EvDocumentInfo *
+ev_document_get_info (EvDocument *document)
+{
+	EvDocumentIface *iface = EV_DOCUMENT_GET_IFACE (document);
+
+	return iface->get_info (document);
+}
+
+char *
+ev_document_get_text (EvDocument  *document,
+		      int          page,
+		      EvRectangle *rect)
+{
+	EvDocumentIface *iface = EV_DOCUMENT_GET_IFACE (document);
+	char *retval;
+
+	LOG ("ev_document_get_text");
+	retval = iface->get_text (document, page, rect);
+
+	return retval;
+}
+
+gboolean
+ev_document_has_attachments (EvDocument *document)
+{
+	EvDocumentIface *iface = EV_DOCUMENT_GET_IFACE (document);
+
+	if (iface->has_attachments == NULL)
+		return FALSE;
+	
+	return iface->has_attachments (document);
+}
+
+GList *
+ev_document_get_attachments (EvDocument *document)
+{
+	EvDocumentIface *iface = EV_DOCUMENT_GET_IFACE (document);
+	GList *retval;
+
+	LOG ("ev_document_get_attachments");
+	if (iface->get_attachments == NULL)
+		return NULL;
+	retval = iface->get_attachments (document);
+
+	return retval;
+}
+
+GdkPixbuf *
+ev_document_render_pixbuf (EvDocument      *document,
+			   EvRenderContext *rc)
+{
+	EvDocumentIface *iface = EV_DOCUMENT_GET_IFACE (document);
+	GdkPixbuf *retval;
+
+	LOG ("ev_document_render_pixbuf");
+	g_assert (iface->render_pixbuf);
+
+	retval = iface->render_pixbuf (document, rc);
+
+	return retval;
 }
 
 void
-ev_document_render (EvDocument  *document,
-		    int          clip_x,
-		    int          clip_y,
-		    int          clip_width,
-		    int          clip_height)
+ev_document_info_free (EvDocumentInfo *info)
 {
-	EvDocumentIface *iface = EV_DOCUMENT_GET_IFACE (document);
-	iface->render (document, clip_x, clip_y, clip_width, clip_height);
+	if (info == NULL)
+		return;
+
+	g_free (info->title);
+	g_free (info->format);
+	g_free (info->author);
+	g_free (info->subject);
+	g_free (info->keywords);
+	g_free (info->security);
+
+	g_free (info);
 }
 
-void
-ev_document_changed (EvDocument *document)
+
+/* Compares two rects.  returns 0 if they're equal */
+#define EPSILON 0.0000001
+
+gint
+ev_rect_cmp (EvRectangle *a,
+	     EvRectangle *b)
 {
-	g_signal_emit (G_OBJECT (document), signals[CHANGED], 0);
-}			    
+	if (a == b)
+		return 0;
+	if (a == NULL || b == NULL)
+		return 1;
+
+	return ! ((ABS (a->x1 - b->x1) < EPSILON) &&
+		  (ABS (a->y1 - b->y1) < EPSILON) &&
+		  (ABS (a->x2 - b->x2) < EPSILON) &&
+		  (ABS (a->y2 - b->y2) < EPSILON));
+}
